<#
  .SYNOPSIS
  This script will install the provided AWSNVMe inf.
  The script will then remove the older version of the driver if exists.
  
  .EXAMPLE
  .\install.ps1
 
#>


#Return codes for dpinst
$RETCODE_ONEPACKAGE_SUCCESSULLY_INSTALLED = 1; 
$RETCODE_TWOPACKAGES_SUCCESSFULLY_INSTALLED =2;
$RETCODE_INSTALLATION_SUCCESS_RESTART_REQUIRED =  1073741825;
$RETCODE_INSTALLATION_SUCCESS_2PACKAGES_RESTART_REQUIRED =  1073741826;
$RETCODE_DRIVER_STORE_COPIED_INSTALL_FAILED = 256;
$RETCODE_INSTALLATION_FAILED =2147549184;
$RETCODE_ONEPACKAGE_INSTALLED_RESTARTREQUIRED_FOR_OTHER =3221291009;
$RETCODE_NO_PACKAGE_FOUND =2147483648;
$RETCODE_REBOOT_REQUIRED_TO_COMPLETE_UNINSTALL = 1073741824;

$Restart = 0
$INFFile =  '.\AWSNVMe.inf'
$AWS_NVME_DEVICE_NAME = "AWS NVMe Instance Storage Adapter"
$ErrorCode = 0;
$UninstallFlags = "/U $INFFile /Q"
$InstallFlags = "/f /a /Q"


# If the Current folder has installed file then do not proceed with installation . This is needed because we dont want to proceed with running the process for the second time if the SSM-Agent
# tries to re run the runcommand request for the second time even though the command is issued only once . I have observed this few times with birdwatcher when we use Restart.
if (Test-Path installed) {
  Write-Host "Detected installed file. The package is already installed.Exiting..." -ForegroundColor Yellow
  exit 0
}


# Verify if the provided argument is indeed a single inf file
if ((Test-Path $INFFile -pathType leaf ) -eq $false) {
	Write-Host 'Error: Invalid Inf input' -foregroundcolor red
	exit 1
}

# Find if any previous versions of the driver are existing.
Write-Host "Checking for Previous Versions of Drivers" -ForegroundColor Green
$uninstallOems = @(Get-WmiObject Win32_PnpSignedDriver | Select DeviceName, DriverVersion, InfName | where {$_.DeviceName -eq $AWS_NVME_DEVICE_NAME} | Select-Object -uniq)
if( $uninstallOems.Length -eq 0)
{
  Write-Host "No Previous Versions of Drivers are Detected.Proceeding with the installation" -ForegroundColor Green
}
else
{
    Write-Host "Previous versions are Detected,Proceeding to Uninstall them." -ForegroundColor Yellow
    $ErrorCode = (Start-Process -FilePath .\dpinst.exe -ArgumentList $UninstallFlags -Wait -Passthru).ExitCode

    if($ErrorCode -eq 0)
    {
        Write-Host "Uninstalled previous versions of drivers Successfully. Proceeding with installing the latest version" -ForegroundColor Green
    }
	elseif($ErrorCode -eq $RETCODE_NO_PACKAGE_FOUND)
	{
		  Write-Host "No DriverStore Entry found.There is no existing AWSNVme to remove"
	}
    elseif($ErrorCode -eq $RETCODE_REBOOT_REQUIRED_TO_COMPLETE_UNINSTALL)
    {
      Write-Host "UnInstall Successful. Machine has to be rebooted to remove the properties from driver store." -ForegroundColor Yellow
      #We will reboot the machine later after install to keep this script simple. This might not be correct but no problems were detected because of this.
      $Restart =1;
    }
    else
    {   
	    #It is observed that Windows 2008R2,removal of old drivers fail sometimes .This might be because they are installed in a different approach.
		#In that case continue installing the new version of drivers .
        Write-Host "Failed to Uninstall the old version of drivers.Continuing to install the latest version ..." -ForegroundColor Red
      
    }
}

$infInfo = Get-ChildItem -Path $INFFile | Select-String "DriverVer" | Select-Object @{n="Driver"; e={$_.FileName.ToLower()}},@{n="Version"; e={$_.Line -Replace "DriverVer=", ""}}
$infInfo | Format-Table -auto
Write-Host "Installing the AWSNVMe of the following version using dpinst $infInfo" -ForegroundColor Green

$ErrorCode = (Start-Process -FilePath .\dpinst.exe -ArgumentList $InstallFlags -Wait -Passthru).ExitCode

if(($ErrorCode -eq  $RETCODE_ONEPACKAGE_SUCCESSULLY_INSTALLED) -or
                      ($ErrorCode -eq  $RETCODE_TWOPACKAGES_SUCCESSFULLY_INSTALLED) -or
                      ($ErrorCode -eq $RETCODE_INSTALLATION_SUCCESS_RESTART_REQUIRED) -or
                      ($ErrorCode -eq $RETCODE_INSTALLATION_SUCCESS_2PACKAGES_RESTART_REQUIRED))
{
    Write-Host "Successfully installed the AWSNVMe" -ForegroundColor Green
    #Create the installed File .
    New-Item installed -ItemType file | out-null
    if($Restart -or  ($ErrorCode -eq $RETCODE_INSTALLATION_SUCCESS_RESTART_REQUIRED) -or ($ErrorCode -eq $RETCODE_INSTALLATION_SUCCESS_2PACKAGES_RESTART_REQUIRED))
    {
      Write-Host "Restarting Computer" -ForegroundColor Green
      Restart-Computer -Force
      exit 0;
    }
}
elseif ($ErrorCode -eq $RETCODE_DRIVER_STORE_COPIED_INSTALL_FAILED)
{
   Write-Host "Driver Copied to DriverStore but installation failed. If you are installing the driver on an instance where the NVMe Device is not present please ignore this error." -ForegroundColor Red
   exit 0;
}
else
{
    Write-Host "Failed to install AWSNVMe.ErrorCode=$ErrorCode" -ForegroundColor Red
    exit 1;
}

